static char *sccsid = "@(#) samples/c/win31/amqsgetw.c, samples, p000          1.3  95/11/02 14:05:56";
 /********************************************************************/
 /*                                                                  */
 /* Program name: AMQSGETW                                           */
 /*                                                                  */
 /* Description: Sample C program, running under Windows 3.X,        */
 /*              that gets messages from a message queue             */
 /*              (example using MQGET)                               */
 /*                                                                  */
 /*                                                                  */
 /* Statement:   Licensed Materials - Property of IBM                */
 /*                                                                  */
 /*              33H2205, 5622-908                                   */ 
 /*              (C) Copyright IBM Corp. 1994, 1995                  */ 
 /*                                                                  */
 /*                                                                  */
 /********************************************************************/
 /*                                                                  */
 /* Function:                                                        */
 /*                                                                  */
 /*                                                                  */
 /*   AMQSGETW is a sample C program, running under Microsoft        */
 /*   Windows(*) 3.X, that gets messages from a message              */
 /*   queue, and is an example of the use of MQGET.                  */
 /*                                                                  */
 /*      -- sample reads from message queue named in the parameter   */
 /*                                                                  */
 /*      -- writes the contents of the message queue to an output    */
 /*         file, assuming each message data to represent a line of  */
 /*         text to be written                                       */
 /*                                                                  */
 /*         messages are removed from the queue                      */
 /*                                                                  */
 /*      -- writes a message to a file specified as a command line   */
 /*         argument, for each MQI reason other than                 */
 /*         MQRC_NONE; stops if there is a MQI completion code       */
 /*         of MQCC_FAILED.                                          */
 /*                                                                  */
 /*                                                                  */
 /*   Program logic:                                                 */
 /*      Parse and verify command line arguments                     */
 /*      MQCONN to specified Queue Manager                           */
 /*      MQOPEN queue for INPUT                                      */
 /*      while no MQI failures,                                      */
 /*      .  MQGET next message, remove from queue                    */
 /*      .  print the result                                         */
 /*      .  (no message available counts as failure, and loop ends)  */
 /*      MQCLOSE the subject queue                                   */
 /*      MQDISC from Queue Manager                                   */
 /*                                                                  */
 /*                                                                  */
 /********************************************************************/
 /*                                                                  */
 /*   AMQSGETW has 3 parameters -                                    */
 /*                 - the output file name         (required)        */
 /*                 - queue manager name           (required)        */
 /*                 - the name of the target queue (optional)        */
 /*                                                                  */
 /********************************************************************/
 /* Trademarks:                                                      */
 /* (*) Windows is a registered trademark of Microsoft Corporation   */
 /********************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
    /* include Windows Header */
 #include <windows.h>
     /* includes for MQI  */
 #include <cmqc.h>

int PASCAL WinMain(HANDLE hInstance,
                   HANDLE hPrevInstance,
                   LPSTR  lpCmdLine,
                   int    nCmdShow)
 {
   /*   Declare file handles for input and output                    */
   FILE    *phOutFile;

   /*   Declare MQI structures needed                                */
   MQOD     od = {MQOD_DEFAULT};    /* Object Descriptor             */
   MQMD     md = {MQMD_DEFAULT};    /* Message Descriptor            */
   MQGMO   gmo = {MQGMO_DEFAULT};   /* get message options           */
      /** note, sample uses defaults where it can **/

   MQHCONN  Hcon;                   /* connection handle             */
   MQHOBJ   Hobj;                   /* object handle                 */
   MQLONG   O_options;              /* MQOPEN options                */
   MQLONG   C_options;              /* MQCLOSE options               */
   MQLONG   CompCode;               /* completion code               */
   MQLONG   OpenCode;               /* MQOPEN completion code        */
   MQLONG   Reason;                 /* reason code                   */
   MQLONG   CReason;                /* reason code for MQCONN        */
   MQBYTE   buffer[101];            /* message buffer                */
   MQLONG   buflen;                 /* buffer length                 */
   MQLONG   messlen;                /* message length received       */
   char     QMName[50];             /* queue manager name            */
   char     QName[50];              /* queue name                    */
   char     OutFileName[256];       /* Output file name              */

   /******************************************************************/
   /*                                                                */
   /*   Parse the command line parameters                            */
   /*                                                                */
   /******************************************************************/
   QMName[0]      = 0;
   QName[0]       = 0;
   OutFileName[0] = 0;

   sscanf( lpCmdLine,
           "%s %s %s",
           OutFileName,
           QMName,
           QName );

   /******************************************************************/
   /*                                                                */
   /*   Verify Command Line Parameters and exit                      */
   /*   if mandatory values are not present                          */
   /*                                                                */
   /******************************************************************/
   /*   Output file name...                                          */
   /******************************************************************/
   if ( strlen( OutFileName ) > 0 )
   {
      phOutFile = fopen( OutFileName, "w" );
      if ( phOutFile == NULL ) 
      {
         exit(99);         
      } 
   } 
   else 
   {
      exit(99);
   } /* endif */

   fprintf( phOutFile, "Sample AMQSGETW start\n");
   fprintf( phOutFile, "Output file \"%s\" opened\n", OutFileName );
   fflush( phOutFile );

   /******************************************************************/
   /*   MQSeries Queue Manager...                                    */
   /******************************************************************/
   if ( strlen( QMName ) == 0 )
   {
      fprintf( phOutFile, "Required parameter missing - queue name\n");
      fprintf( phOutFile, "Required parameters are:\n");
      fprintf( phOutFile, "\tOutput File name\n");
      fprintf( phOutFile, "\tQueue Manager name\n");
      fprintf( phOutFile, "\tQueue name (Optional - will use default message queue)\n");
      fprintf( phOutFile, "Sample AMQSGETW end\n" );
      fclose( phOutFile );
      exit(99);
   }

   fprintf( phOutFile, "Queue Manager \"%s\" will be used\n", QMName );

   if ( strlen( QName ) == 0)
      fprintf( phOutFile, "No parameter for Queue Name. Default Queue Name will be used\n" );
   else 
      fprintf( phOutFile, "Queue Name \"%s\" will be used\n", QName );
   fflush( phOutFile );

   /******************************************************************/
   /*                                                                */
   /*   Create object descriptor for subject queue                   */
   /*                                                                */
   /******************************************************************/
   strcpy(od.ObjectName, QName);

   /******************************************************************/
   /*                                                                */
   /*   Connect to queue manager                                     */
   /*                                                                */
   /******************************************************************/
   MQCONN(QMName,                  /* queue manager                  */
          &Hcon,                   /* connection handle              */
          &CompCode,               /* completion code                */
          &CReason);               /* reason code                    */

   /* report reason and stop if it failed     */
   if (CompCode == MQCC_FAILED)
   {
     fprintf( phOutFile, "MQCONN ended with reason code %ld\n", CReason);
     fclose( phOutFile );
     exit(99);
   }

   /******************************************************************/
   /*                                                                */
   /*   Open the named message queue for input; exclusive or shared  */
   /*   use of the queue is controlled by the queue definition here  */
   /*                                                                */
   /******************************************************************/
   O_options = MQOO_INPUT_AS_Q_DEF   /* open queue for input         */
         + MQOO_FAIL_IF_QUIESCING;   /* but not if MQM stopping      */
   MQOPEN(Hcon,                      /* connection handle            */
          &od,                       /* object descriptor for queue  */
          O_options,                 /* open options                 */
          &Hobj,                     /* object handle                */
          &OpenCode,                 /* completion code              */
          &Reason);                  /* reason code                  */

   /* report reason, if any; stop if failed      */
   if (Reason != MQRC_NONE)
   {
     fprintf( phOutFile, "MQOPEN ended with reason code %ld\n", Reason);
   }

   if (OpenCode == MQCC_FAILED)
   {
     fprintf( phOutFile, "unable to open queue for input\n");
   }

   /******************************************************************/
   /*                                                                */
   /*   Get messages from the message queue                          */
   /*   Loop until there is a failure                                */
   /*                                                                */
   /******************************************************************/
   CompCode = OpenCode;       /* use MQOPEN result for initial test  */
   while (CompCode != MQCC_FAILED)
   {
     buflen = sizeof(buffer) - 1; /* buffer size available for GET   */
     gmo.Options = MQGMO_WAIT     /* wait for new messages           */
                  + MQGMO_CONVERT;/* convert if necessary            */
     gmo.WaitInterval = 15000;    /* 15 second limit for waiting     */

     /****************************************************************/
     /*                                                              */
     /*   In order to read the messages in sequence, MsgId and       */
     /*   CorrelID must have the default value.  MQGET sets them     */
     /*   to the values in for message it returns, so re-initialise  */
     /*   them before every call                                     */
     /*                                                              */
     /****************************************************************/
     memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
     memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));

     MQGET(Hcon,                /* connection handle                 */
           Hobj,                /* object handle                     */
           &md,                 /* message descriptor                */
           &gmo,                /* get message options               */
           buflen,              /* buffer length                     */
           buffer,              /* message buffer                    */
           &messlen,            /* message length                    */
           &CompCode,           /* completion code                   */
           &Reason);            /* reason code                       */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       if (Reason == MQRC_NO_MSG_AVAILABLE)
       {                         /* special report for normal end    */
         fprintf( phOutFile, "no more messages\n");
       }
       else                      /* general report for other reasons */
       {
         fprintf( phOutFile, "MQGET ended with reason code %ld\n", Reason);

         /*   treat truncated message as a failure for this sample   */
         if (Reason == MQRC_TRUNCATED_MSG_FAILED)
         {
           CompCode = MQCC_FAILED;
         }
       }
     }

     /****************************************************************/
     /*   Display each message received                              */
     /****************************************************************/
     if (CompCode != MQCC_FAILED)
     {
       buffer[messlen] = '\0';            /* add terminator          */
       fprintf( phOutFile, "MQGET OK - message contents: <%s>\n", buffer );
     }
   }

   /******************************************************************/
   /*                                                                */
   /*   Close the source queue (if it was opened)                    */
   /*                                                                */
   /******************************************************************/
   if (OpenCode != MQCC_FAILED)
   {
     C_options = 0;                   /* no close options            */
     MQCLOSE(Hcon,                    /* connection handle           */
             &Hobj,                   /* object handle               */
             C_options,
             &CompCode,               /* completion code             */
             &Reason);                /* reason code                 */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       fprintf( phOutFile, "MQCLOSE ended with reason code %ld\n", Reason);
     }
   }

   /******************************************************************/
   /*                                                                */
   /*   Disconnect from MQM if not already connected                 */
   /*                                                                */
   /******************************************************************/
   if (CReason != MQRC_ALREADY_CONNECTED )
   {
     MQDISC(&Hcon,                     /* connection handle          */
            &CompCode,                 /* completion code            */
            &Reason);                  /* reason code                */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       fprintf( phOutFile, "MQDISC ended with reason code %ld\n", Reason);
     }
   }

   /******************************************************************/
   /*                                                                */
   /*   Clean up open files                                          */
   /*                                                                */
   /******************************************************************/
   fprintf( phOutFile, "Sample AMQSGETW end\n");
   fclose( phOutFile );

   /******************************************************************/
   /*                                                                */
   /* END OF AMQSGETW                                                */
   /*                                                                */
   /******************************************************************/
   return(0);
 }
